# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Dostupné také v",
    "Also available in:": "Dostupné také v",
    "Archive": "Archiv",
    "Categories": "Kategorie",
    "LANGUAGE": "Čeština",
    "More posts about %s": "Další příspěvky o %s",
    "More posts about": "Další příspěvky o",
    "Newer posts": "Novější příspěvky",
    "Next post": "Další příspěvek",
    "Older posts": "Starší příspěvky",
    "Original site": "Původní stránka",
    "Posted": "Zveřejněno",
    "Posted:": "Zveřejněno:",
    "Posts about %s": "Příspěvky o %s",
    "Posts for year %s": "Příspěvky v roce %s",
    "Posts for {month} {year}": "Příspěvky v {month} {year}",
    "Previous post": "Předchozí příspěvek",
    "Read in English": "Číst v češtině",
    "Read more": "Číst dál",
    "Source": "Zdroj",
    "Tags and Categories": "Štítky a kategorie",
    "Tags": "Štítky",
    "old posts, page %d": "staré příspěvky, strana %d",
    "page %d": "strana %d",
}
