# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "他の言語で読む",
    "Also available in:": "他の言語で読む：",
    "Archive": "過去の記事",
    "Categories": "カテゴリー",
    "LANGUAGE": "日本語",
    "More posts about %s": "タグ： %s",
    "More posts about": "タグ：",
    "Newer posts": "新しい記事",
    "Next post": "次の記事",
    "Older posts": "過去の記事",
    "Original site": "元のサイト",
    "Posted": "投稿日時",
    "Posted:": "投稿日時：",
    "Posts about %s": "%sについての記事",
    "Posts for year %s": "%s年の記事",
    "Posts for {month} {year}": "{year}年{month}月の記事",
    "Previous post": "前の記事",
    "Read in English": "日本語で読む",
    "Read more": "続きを読む",
    "Source": "ソース",
    "Tags and Categories": "タグとカテゴリー",
    "Tags": "タグ",
    "old posts, page %d": "前の記事 %dページ目",
    "page %d": "",
}
