# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Also available in",
    "Also available in:": "Also available in:",
    "Archive": "Archive",
    "Categories": "Categories",
    "LANGUAGE": "English",
    "More posts about %s": "More posts about %s",
    "More posts about": "More posts about",
    "Newer posts": "Newer posts",
    "Next post": "Next post",
    "Older posts": "Older posts",
    "Original site": "Original site",
    "Posted": "Posted",
    "Posted:": "Posted:",
    "Posts about %s": "Posts about %s",
    "Posts for year %s": "Posts for year %s",
    "Posts for {month} {year}": "Posts for {month} {year}",
    "Previous post": "Previous post",
    "Read in English": "Read in English",
    "Read more": "Read more",
    "Source": "Source",
    "Tags and Categories": "Tags and Categories",
    "Tags": "Tags",
    "old posts, page %d": "old posts, page %d",
    "page %d": "page %d",
}
