# -*- coding: utf-8 -*-

# Copyright © 2012-2014 Daniel Devine and others.

# Permission is hereby granted, free of charge, to any
# person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the
# Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice
# shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from nikola.plugin_categories import SignalHandler
from blinker import signal
import logbook


class SmtpHandler(SignalHandler):
    name = 'smtp'

    def attach_handler(self, sender):
        """Add the handler to a list of handlers that are attached when get_logger() is called.."""
        smtpconf = self.site.config.get('LOGGING_HANDLERS').get('smtp')
        if smtpconf:
            smtpconf['format_string'] = '''\
Subject: {record.level_name}: {record.channel}

{record.message}
'''
            self.site.loghandlers.append(logbook.MailHandler(
                smtpconf.pop('from_addr'),
                smtpconf.pop('recipients'),
                **smtpconf
            ))

    def set_site(self, site):
        self.site = site

        ready = signal('sighandlers_loaded')
        ready.connect(self.attach_handler)
