# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "También disponible en",
    "Also available in:": "También disponible en:",
    "Archive": "Archivo",
    "Categories": "Categorías",
    "LANGUAGE": "Español",
    "More posts about %s": "Más posts sobre %s",
    "More posts about": "Más posts sobre",
    "Newer posts": "Posts posteriores",
    "Next post": "Siguiente post",
    "Older posts": "Posts anteriores",
    "Original site": "Sitio original",
    "Posted": "Publicado",
    "Posted:": "Publicado:",
    "Posts about %s": "Posts sobre %s",
    "Posts for year %s": "Posts del año %s",
    "Posts for {month} {year}": "Posts de {month} {year}",
    "Previous post": "Post anterior",
    "Read in English": "Leer en español",
    "Read more": "Leer más",
    "Source": "Código",
    "Tags and Categories": "Tags y Categorías",
    "Tags": "Tags",
    "old posts, page %d": "posts antiguos, página %d",
    "page %d": "página %d",
}
