# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Também disponível em",
    "Also available in:": "Também disponível em:",
    "Archive": "Arquivo",
    "Categories": "Categorias",
    "LANGUAGE": "Português",
    "More posts about %s": "Mais posts sobre %s",
    "More posts about": "Mais posts sobre",
    "Newer posts": "Posts mais recentes",
    "Next post": "Próximo post",
    "Older posts": "Posts mais antigos",
    "Original site": "Site original",
    "Posted": "Publicado",
    "Posted:": "Publicado:",
    "Posts about %s": "Posts sobre %s",
    "Posts for year %s": "Posts do ano %s",
    "Posts for {month} {year}": "Posts de {month} {year}",
    "Previous post": "Post anterior",
    "Read in English": "Ler em português",
    "Read more": "Leia mais",
    "Source": "Código",
    "Tags and Categories": "Tags e Categorias",
    "Tags": "Tags",
    "old posts, page %d": "Posts antigos, página %d",
    "page %d": "página %d",
}
