# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "També disponibles en",
    "Also available in:": "També disponibles en:",
    "Archive": "Arxiu",
    "Categories": "",
    "LANGUAGE": "Català",
    "More posts about %s": "Més entrades sobre %s",
    "More posts about": "Més entrades sobre",
    "Newer posts": "Entrades posteriors",
    "Next post": "Entrada següent",
    "Older posts": "Entrades anteriors",
    "Original site": "Lloc original",
    "Posted": "Publicat",
    "Posted:": "Publicat:",
    "Posts about %s": "Entrades sobre %s",
    "Posts for year %s": "Entrades de l'any %s",
    "Posts for {month} {year}": "",
    "Previous post": "Entrada anterior",
    "Read in English": "Llegeix-ho en català",
    "Read more": "Llegeix-ne més",
    "Source": "Codi",
    "Tags and Categories": "",
    "Tags": "Etiquetes",
    "old posts, page %d": "entrades antigues, pàgina %d",
    "page %d": "pàgina %d",
}
