# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "ان زبانوں میں بھی دستیاب",
    "Also available in:": "ان زبانوں میں بھی دستیاب:",
    "Archive": "آرکائیو",
    "Categories": "زمرے",
    "LANGUAGE": "اردو",
    "More posts about %s": "%s کے بارے میں مزید تحاریر",
    "More posts about": " کے بارے میں مزید تحاریر",
    "Newer posts": "نئی تحاریر",
    "Next post": "اگلی تحریر",
    "Older posts": "پرانی تحاریر",
    "Original site": "اصلی سائٹ",
    "Posted": "اشاعت",
    "Posted:": "اشاعت:",
    "Posts about %s": "%s کے بارے میں تحاریر",
    "Posts for year %s": "سال %s کی تحاریر",
    "Posts for {month} {year}": "{month} {year} کی تحاریر",
    "Previous post": "پچھلی تحریر",
    "Read in English": "اردو میں پڑھیے",
    "Read more": "مزید پڑھیے",
    "Source": "سورس",
    "Tags and Categories": "ٹیگز اور زمرے",
    "Tags": "ٹیگز",
    "old posts, page %d": "پرانی تحاریر صفحہ %d",
    "page %d": "",
}
